@@ -124,12 +124,16 @@ class Worker:
         for job in self.cron_jobs:
             if not croniter.is_valid(job.cron):
                 raise ValueError(f"Cron is invalid {job.cron}")
-            functions.add(job.function)
+            # functions.add(job.function)
+            self.functions[job.function.__qualname__] = job.function
+            # self.logger(kind="startup").info(f"Added cron job {job.function.__qualname__} with cron {job.cron} to worker {self.name}.")
 
         for function in functions:
             if isinstance(function, tuple): name, function = function
+            # elif isinstance(function, Job):
             else: name = function.__qualname__
             self.functions[name] = function
+        # self.logger(kind="startup").info(f"Added {len(self.functions)} functions to worker {self.name}: {self.functions.keys()}")
     
 
     def logger(self, job: 'Job' = None, kind: str = "enqueue"):
@@ -343,12 +347,12 @@ class Worker:
             job.attempts += 1
             await job.update()
             context = {**self.context, "job": job}
+            # self.logger(job = None, kind = "process").error(f"JOB: {job}")
             await self._before_process(context)
             if job.function not in self.silenced_functions:
                 self.logger(job = job, kind = "process").info(
                     f"← duration={job.duration('running')}ms, node={self.node_name}, func={job.function}"
                 )
-                
 
             function = ensure_coroutine_function(self.functions[job.function])
             task = asyncio.create_task(function(context, **(job.kwargs or {})))
@@ -449,7 +453,7 @@ class Worker:
     def add_context(
         obj: typing.Optional[typing.Any] = None,
         name: typing.Optional[str] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         silenced: typing.Optional[bool] = None,
         _fx: typing.Optional[typing.Callable] = None,
         **kwargs,
@@ -478,7 +482,7 @@ class Worker:
     def add_dependency(
         obj: typing.Optional[typing.Any] = None,
         name: typing.Optional[str] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         _fx: typing.Optional[typing.Callable] = None,
         silenced: typing.Optional[bool] = None,
         **kwargs,
@@ -499,7 +503,7 @@ class Worker:
     @staticmethod
     def on_startup(
         name: typing.Optional[str] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         _fx: typing.Optional[typing.Callable] = None,
         silenced: typing.Optional[bool] = None,
         **kwargs,
@@ -531,7 +535,7 @@ class Worker:
     @staticmethod
     def on_shutdown(
         name: typing.Optional[str] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         _fx: typing.Optional[typing.Callable] = None,
         silenced: typing.Optional[bool] = None,
         **kwargs,
@@ -547,7 +551,7 @@ class Worker:
     def add_function(
         name: typing.Optional[str] = None,
         _fx: typing.Optional[typing.Callable] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         silenced: typing.Optional[bool] = None,
         **kwargs,
     ):
@@ -571,7 +575,7 @@ class Worker:
     def add_cronjob(        
         schedule: typing.Optional[typing.Union[typing.Dict, typing.List, str]] = None,
         _fx: typing.Optional[typing.Callable] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         silenced: typing.Optional[bool] = None,
         **kwargs,
     ):
@@ -591,7 +595,7 @@ class Worker:
         task: TaskType = TaskType.default,
         name: typing.Optional[str] = None,
         obj: typing.Optional[typing.Any] = None,
-        verbose: typing.Optional[bool] = False,
+        verbose: typing.Optional[bool] = None,
         **kwargs,
     ):
         """
